############################  SMiP 2024 MLM Example 3  #########################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

# To estimate MLMs using lmer
# Re-install to fix problem of matrix incompatibility
#oo <- options(repos = "https://cran.r-project.org/")
#utils::install.packages("Matrix")
#utils::install.packages("lme4")
#options(oo)
library(Matrix); library(lme4)

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 3             #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/Papers and Data/SMiP/SMiP_2024_MLM_Part2/"
filename = "Example3_Data.xlsx"
setwd(dir=filesave)

#  Import long-format example excel data file from sheet "long"
Example3 = read_excel(paste0(filesave,filename), sheet="long") 
# Convert to data frame to use in analysis
Example3 = as.data.frame(Example3)

# Filter to only cases complete on all variables to be used below
Example3 = Example3[complete.cases(Example3[ , 
                    c("correct","clutter","relevant","bright","sign","age75","visimp")]),]

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example3_R_Output.txt"))

# Labels for original variables in comments only
#PersonID=  "PersonID: Person Identifier"
#PictureID= "PictureID: Picture Identifier"
#correct=   "correct: Correct Response (0=no, 1=yes)"
#RTsec=     "RTsec: Response Time in Seconds if Correct"
#clutter=   "clutter: Rated Clutter in Picture Z-Score"
#relevant=  "relevant: Change Relevant to Driving (0=no, 1=yes)"
#bright=    "bright: Rated Change Brightness Z-Score"
#sign=      "sign: Change to Legible Sign (0=no, 1=yes)"
#age75=     "age75: Age in Years (0=75)"
#visimp=    "visimp: Vision Factor Z-Score"

print("R Descriptive Statistics for Example Variables")
print(describe(x=Example3[ , c("correct","clutter","relevant","bright","sign","age75","visimp")], 
         fast=TRUE), digits=3)


################################################################################
#####              BEGIN EXAMPLE 7 BINARY CORRECT MODELS                   #####
################################################################################

# NOTE: I am using Laplace estimation in anticipation of crossed random effects

print("R Model 1: Single-Level Empty Means for Binary Correct Response")
Model1 = glm(data=Example3, family=binomial(link="logit"), formula=correct~1)
summary(Model1) # residual deviance = -2LL already
print("Convert logits to probability via shorter inverse link")
Model1Prob=1/(1+exp(-1*coefficients(Model1))); Model1Prob 

txtComment(" ") # insert blank space in output


print("R Model 2: Random Persons Only, Empty Means")
Model2 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~1+(1|PersonID))
print("Show -2LL with more precision, results, and ICC using 3.29=residual variance")
-2*logLik(Model2); summary(Model2); icc(Model2)
print("LRT for person random intercept"); anova(Model2, Model1)
print("Convert logits to probability via shorter inverse link")
Model2Prob=1/(1+exp(-1*fixef(Model2))); Model2Prob 

txtComment(" ") # insert blank space in output


print("R Model 3: Random Persons Only, Rasch Fixed Items")
print("Rasch version of 1PL: Person Trait Variance Estimated, Single Discrimination=1")
Model3 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~0+as.factor(PictureID)+(1|PersonID))
print("Show -2LL with more precision and results")
-2*logLik(Model3); summary(Model3); 
print("LRT for fixed item differences"); anova(Model3, Model2)
print("Compute predicted mean per item given theta=0")
Model3Prob = 1/(1+exp(-1*(as.data.frame(fixef(Model3))))); Model3Prob

txtComment(" ") # insert blank space in output


print("R Model 4: Random Persons, Empty Means Random Items")
Model4 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~1+(1|PersonID)+(1|PictureID))
print("Show -2LL with more precision, results, and ICCs")
-2*logLik(Model4); summary(Model4); icc(Model4, by_group=TRUE)
print("LRT for item random intercept"); anova(Model4, Model2)

# Save each variance as an object to compute pseudo-R2
Model4SubIntVar  = as.data.frame(VarCorr(Model4))[1,4]
Model4ItemIntVar = as.data.frame(VarCorr(Model4))[2,4]

txtComment(" ") # insert blank space in output


print("R Model 5: Random Persons, Predicted Random Items")
Model5 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~1+clutter+relevant+bright+sign+(1|PersonID)+(1|PictureID))
print("Show -2LL with more precision and results")
-2*logLik(Model5); summary(Model5)

# Save each variance as an object to compute pseudo-R2
Model5SubIntVar  = as.data.frame(VarCorr(Model5))[1,4]
Model5ItemIntVar = as.data.frame(VarCorr(Model5))[2,4]

# Compute pseudo-R2 for each variance
Model5SubIntR2  = (Model4SubIntVar- Model5SubIntVar) /Model4SubIntVar;  Model5SubIntR2
Model5ItemIntR2 = (Model4ItemIntVar-Model5ItemIntVar)/Model4ItemIntVar; Model5ItemIntR2

txtComment(" ") # insert blank space in output


print("R Model 6: Random Persons Only, Predicted Fixed Items")
Model6 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~1+clutter+relevant+bright+sign+(1|PersonID))
print("Show -2LL with more precision and results")
-2*logLik(Model6); summary(Model6)
print("LRT for random item variance leftover"); anova(Model5, Model6)

txtComment(" ") # insert blank space in output

#################################################
# Correspondence of item parms across models
# Model 3 Rasch Fixed Items
Fixed_b = as.data.frame(fixef(Model3))
# Model 4 Random Item 
Random_b = as.data.frame(ranef(Model4)$PictureID) + (fixef(Model4))
# Model 6 Predicted Fixed Items
Pred = as.data.frame(unique(Example3[,c("PictureID","clutter","relevant","bright","sign")]))   
Fixed_Pred_b = as.data.frame(predict(Model6, newdata=Pred, re.form=NA, type="link"))
# Model 5 Predicted Random Items (random intercept not included)
Random_Pred_b = as.data.frame(predict(Model5, newdata=Pred, re.form=NA, type="link"))
# Combine results and rename
Compare_b = cbind(Fixed_b, Random_b, Fixed_Pred_b, Random_Pred_b)
colnames(Compare_b) = c("Fixed_b","Random_b","Fixed_Pred_b","Random_Pred_b")
print("Correlation of item easiness across models")
print(cor(Compare_b), digits=3)
##################################################

txtComment(" ") # insert blank space in output


print("R Model 7: Predicted Random Persons, Predicted Random Items")
Model7 = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
               correct~1+clutter+relevant+bright+sign+age75+visimp+(1|PersonID)+(1|PictureID))
print("Show -2LL with more precision and results")
-2*logLik(Model7); summary(Model7)

# Save each variance as an object to compute pseudo-R2
Model7SubIntVar  = as.data.frame(VarCorr(Model7))[1,4]
Model7ItemIntVar = as.data.frame(VarCorr(Model7))[2,4]

# Compute pseudo-R2 for each variance
Model7SubIntR2  = (Model4SubIntVar- Model7SubIntVar) /Model4SubIntVar;  Model7SubIntR2
Model7ItemIntR2 = (Model4ItemIntVar-Model7ItemIntVar)/Model4ItemIntVar; Model7ItemIntR2

txtComment(" ") # insert blank space in output


print("R Model 8a: Add Random Brightness Slope over Persons (NS)")
Model8a = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
                correct~1+clutter+relevant+bright+sign+age75+visimp
                         +(1+bright|PersonID)+(1|PictureID))
print("Show -2LL with more precision and results")
-2*logLik(Model8a); summary(Model8a)
print("LRT for random brightness slope over persons"); anova(Model8a, Model7)

print("LRT for random slope variance using a mixture-chi-square test")
Model8aDiff2LL = -2*(logLik(Model7)-logLik(Model8a))
Model8aDiffP1 = pchisq(Model8aDiff2LL, df=1, lower.tail=FALSE)
Model8aDiffP2 = pchisq(Model8aDiff2LL, df=2, lower.tail=FALSE)
Model8aDiffP12 = (.5*Model8aDiffP1) + (.5*Model8aDiffP2)
print("Test statistic and mixture p-values for df=1,2") 
Model8aDiff2LL; Model8aDiffP12

# Models 8b, 8c, and 8d: None of the other item predictors had signif random slopes either

txtComment(" ") # insert blank space in output


print("R Model 9a: Add Random Age Slope over Items (NS)")
Model9a = glmer(data=Example3, family=binomial(link="logit"), nAGQ=1,
                correct~1+clutter+relevant+bright+sign+age75+visimp
                +(1|PersonID)+(1+age75|PictureID))
print("Show -2LL with more precision and results")
-2*logLik(Model9a); summary(Model9a)
print("LRT for random age slope over items"); anova(Model9a, Model7)

print("LRT for random slope variance using a mixture-chi-square test")
Model9aDiff2LL = -2*(logLik(Model7)-logLik(Model9a))
Model9aDiffP1 = pchisq(Model9aDiff2LL, df=1, lower.tail=FALSE)
Model9aDiffP2 = pchisq(Model9aDiff2LL, df=2, lower.tail=FALSE)
Model9aDiffP12 = (.5*Model9aDiffP1) + (.5*Model9aDiffP2)
print("Test statistic and mixture p-values for df=1,2") 
Model9aDiff2LL; Model9aDiffP12

txtComment(" ") # insert blank space in output


print("R Model 7 Probit Scale for Mplus: Predicted Random Persons, Predicted Random Items")
Model7p = glmer(data=Example3, family=binomial(link="probit"), nAGQ=1,
                correct~1+clutter+relevant+bright+sign+age75+visimp+(1|PersonID)+(1|PictureID))
print("Show -2LL with more precisipon and results")
-2*logLik(Model7p); summary(Model7)

txtComment(" ") # insert blank space in output


############################################################################
if (!require("mirt")) install.packages("mirt")
library(mirt) # To estimate IRT models using full-info ML
# Note that mirt masks "fixef" from glmer, so I ran it last

# IRT models using wide data

#  Import wide-format example excel data file from sheet "wide"
Example3_wide = read_excel(paste0(filesave,filename), sheet="wide") 
# Convert to data frame to use in analysis
Example3_wide = as.data.frame(Example3_wide)

# Remove ID column so it doesn't get included in the model as an outcome
Example3_wide = subset(Example3_wide, select=-c(PersonID,age75,visimp))

print("R Model 3 again: IRT Rasch -- Item Easiness Instead of Difficulty")
print("Rasch version constrains single discrimination to 1 and estimates trait variance")
MIRTmodel = mirt(data=Example3_wide, model=1, guess=0, upper=1, itemtype="Rasch")
print("Show MIRT results with -2LL and item easiness parameters")
show(MIRTmodel); -2*logLik(MIRTmodel); coef(MIRTmodel)

txtComment(" ") # insert blank space in output

#######################################################################################
# Close external file of saved results
txtStop()


